; clock.asm
; CPU clock speed test for TI-84+CSE
; Only accurate to +/- 50 kHz or so
; critor ran this and got
; 13960 33651 33669 33686
; = 6.30 15.19 15.20 15.21 MHz
; http://tiplanet.org/forum/gallery/image.php?album_id=144&image_id=2100&view=no_count
; numOfTicks should have been changed to like 64 or 32 for support for reading
; speeds greater than 16 MHz

.nolist
#include "ti84pcse.inc"
.list

;===============================================================================
;====== Actual Program Code ====================================================
;===============================================================================

;====== Program Equates ========================================================
numOfTicks	.equ	254; 32



;====== Program ================================================================

	.org	userMem - 2
	.db	tExtTok, tAsm84CCmp
	
	ld	hl, startMsg
	b_call(_PutS)
	;b_call(_NewLine)
	xor	a	; cpu6MHz
	out	(pMemDelay), a	; Erase any memory access delays
	out	(pCpuSpeed), a
	call	findCPUSpeed
	ld	a, cpu15MHz
	out	(pCpuSpeed), a
	call	findCPUSpeed
	ld	a, cpu20MHz
	out	(pCpuSpeed), a
	call	findCPUSpeed
	ld	a, cpu25MHz
	out	(pCpuSpeed), a
	call	findCPUSpeed
	ret


findCPUSpeed:



speedTest:
; This uses the crystal timers to time a loop which takes a known amount of CPU 
; clock cycles.  This is then used to calculate the CPU clock speed.  If you 
; make the loop longer or shorter, remember to update the section below that
; computes the speed or you might get some wild results!  The loop should be as
; short as possible for better results.
	di
	ld	de, $0
;	ld	hl, 8000h
	ld	ix, 8001h
	ld	a, tmr2048Hz			; 2048 Hz
	out	(pCrstlTmr1Freq), a
	xor	a
	out	(pCrstlTmr1Cfg), a
	ld	a, numOfTicks
	out	(pCrstlTmr1Count), a
speedLoop:
	inc	de				;  6
						; 0
	;ld	(hl), 0				; 10
	ld	(ix+0), 0			; 19
	in	a, (pIntId)			; 11
	bit	5, a				;  8
	jr	z, speedLoop			; 12
		   				; 37 cycles
						; or 47 or 56 cycles
						; Currently 56
	xor	a
	out	(pCrstlTmr1Cfg), a
	out	(pCrstlTmr1Freq), a
	ei
; Compute speed
	ex	de, hl
	b_call(_SetXXXXOP2)
	ld	hl, factor
	rst	rMOV9TOOP1
	b_call(_FPMult)
	ld	a, 6;13
	b_call(_FormEReal)
	ld	hl, OP3
	b_call(_PutS)
	ld	hl, MHzText
	b_call(_PutS)
	b_call(_NewLine)
	ret
factor:
	.db	00, 7Ch, 45h, 15h, 29h, 00, 00, 00, 00, 00, 00


#ifdef NO_LONGER_USED
;====== Routines ===============================================================
DispDec:
	ld	bc,-10000
	call	Num1
	ld	bc,-1000
	call	Num1
	ld	bc,-100
	call	Num1
	ld	c,-10
	call	Num1
	ld	c,b

Num1	ld	a,'0'-1
Num2	inc	a
	add	hl,bc
	jr	c,Num2
	sbc	hl,bc
	b_call(_PutC)
	ret

;------ ------------------------------------------------------------------------
DispHL:
	ld	a, h
	call	DispByte
	ld	a, l
;------ ------------------------------------------------------------------------
DispByte:
; Displays a byte in hex.
; A is the byte to display.
	push	af
	push	bc
	push	af
	srl	a
	srl	a
	srl	a
	srl	a
	call	db1
	pop	af
;	ld	b, 00001111b
	and	15;b
	call	db1
	pop	bc
	pop	af
	ret
db1:	ld	b, '0'
	cp	0Ah
	jr	c, db2
	ld	b, 'A'-0Ah
db2:	add	a, b
	b_call(_PutC)
	ret
#endif

;====== Data ===================================================================
		;12345678901234567890123456
startMsg:
	.db	"Starting CPU speed test..."
	.db	0
MHzText:
	.db	" MHz"
	.db	0

.end
.end
